using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

using GRV11; // Requires GRV11.cs source code or a "reference" to GRV11DLL.dll

namespace ClientDrw
{
    public partial class Form1 : Form
    {
        IPEndPoint FIPEndPoint = null;
        UdpClient FServer = null;
        Thread FThread = null;

        public GRControl mGRControl;
        public Handler1 mGRExampleHandler1;
        private System.Windows.Forms.Timer mTimer;

        public Form1()
        {
            InitializeComponent();

            this.mGRExampleHandler1 = new Handler1(this, mGRControl);
            this.mGRControl.OpenGLStarted += new GRControl.DelegateOpenGLStarted(this.mGRExampleHandler1.OpenGLStarted);
            this.mGRControl.KeyDown += new KeyEventHandler(this.mGRExampleHandler1.KeyDown);
            this.mGRControl.KeyUp += new KeyEventHandler(this.mGRExampleHandler1.KeyUp);
            this.mGRControl.MouseDown += new MouseEventHandler(this.mGRExampleHandler1.MouseDown);
            this.mGRControl.MouseUp += new MouseEventHandler(this.mGRExampleHandler1.MouseUp);
            this.mGRControl.MouseMove += new MouseEventHandler(this.mGRExampleHandler1.MouseMove);
            this.mGRControl.MouseWheel += new MouseEventHandler(this.mGRExampleHandler1.MouseWheel);
            //        this.mGRControl.Paint += new PaintEventHandler(this.mGRExampleHandler1.Paint);
            this.mGRControl.KeyPress += new KeyPressEventHandler(this.mGRExampleHandler1.KeyPress);

            this.mTimer = new System.Windows.Forms.Timer();
            this.mTimer.Interval = 10; // 10-millisecond timer
            this.mTimer.Tick += new EventHandler(PrivateTimerTickEventHandler);
            this.mTimer.Start();

            // Set focus to the control so that it can immediately accept input
            this.mGRControl.Focus();

            // Also, whenever the form becomes activated, set focus to the main
            // control on the form.  The following sets up an event handler for
            // that purpose.
            this.Activated += new EventHandler(PrivateActivatedEventHandler);

            // We want to preview dialog keys (most importantly, the cursor
            // keys: up, down, right, left) so we can forward such events to
            // the appropriate child control.
            this.KeyPreview = true;
        }

        void PrivateTimerTickEventHandler(object sender, EventArgs e)
        {
            if (false == DesignMode)
            {
                this.mGRControl.Invalidate();
            }
        }

        private void PrivateActivatedEventHandler(object sender, EventArgs e)
        {
            // When this form becomes activated, after some time of not
            // being active, set input focus to the main control on the form.
            this.mGRControl.Focus();
        }

        // Cursor keys (up,down,left,right) need to be specially captured
        // and forwarded to the control.
        // CAUTION: The KeyPreview property of this Form must be set to 'true' 
        // for the following method to be called.
        protected override bool ProcessDialogKey(Keys keyData)
        {
            if
                (
                   (keyData == Keys.Up)
                || (keyData == Keys.Down)
                || (keyData == Keys.Left)
                || (keyData == Keys.Right)
                )
            {
                KeyEventArgs e = new KeyEventArgs(keyData);
                this.mGRExampleHandler1.KeyDown(this.mGRControl, e);
                return (true);
            }

            return base.ProcessDialogKey(keyData);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            FIPEndPoint = new IPEndPoint(IPAddress.Any, 9999);
            FServer = new UdpClient(9999);
            FServer.EnableBroadcast = true;

            FThread = new Thread(new ThreadStart(StartServer));
            FThread.IsBackground = true;
            FThread.Start();
        }

        public void StartServer()
        {
            while (true)
            {
                try
                {
                    int FPS = System.Environment.TickCount;

                    byte[] LData = FServer.Receive(ref FIPEndPoint);

                    LData = Convert.FromBase64String(new ASCIIEncoding().GetString(LData));
                    if (LData.Length >= 4)
                    {
                        if (new ASCIIEncoding().GetString(LData, 0, 3).Equals("MTR"))
                        {
                            byte LLength = LData[3];
                            if (LData.Length >= CCube.ByteSize * LLength + 4)
                            {
                                if ((mGRExampleHandler1.FCubes == null) || (mGRExampleHandler1.FCubes.Length != LLength))
                                {
                                    mGRExampleHandler1.FCubes = new CCube[LLength];

                                    for (int LIndex = 0; LIndex < LLength; LIndex++)
                                    {
                                        mGRExampleHandler1.FCubes[LIndex] = new CCube();
                                    }
                                }

                                int LPos = 4;
                                foreach (CCube LCube in mGRExampleHandler1.FCubes)
                                {
                                    LCube.SetBytes(LData, LPos);
                                    LPos += CCube.ByteSize;
                                }

                                //mGRExampleHandler1.Paint(mGRControl, null);
                                this.Invoke(new EventHandler(delegate(object sender2, EventArgs args2)
                                {
                                    mGRExampleHandler1.Paint(mGRControl, null);
                                }));
                            }

                            this.Invoke(new EventHandler(delegate(object sender2, EventArgs args2)
                            {
                                FPS = 1000 / (System.Environment.TickCount - FPS + 1);
                                this.Text += " FPS=" + FPS.ToString();
                            }));
                        }
                    }
                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message);
                }
            }
        }
    }
}